@@ -31,6 +31,7 @@ import org.apache.commons.lang3.ArrayUtils;
 import com.google.protobuf.ByteString;
 
 import db.Transaction;
+import generic.theme.GIcon;
 import ghidra.app.plugin.core.debug.disassemble.DebuggerDisassemblerPlugin;
 import ghidra.app.plugin.core.debug.disassemble.TraceDisassembleCommand;
 import ghidra.app.services.DebuggerControlService;
@@ -44,6 +45,7 @@ import ghidra.framework.model.*;
 import ghidra.framework.plugintool.AutoService;
 import ghidra.framework.plugintool.AutoService.Wiring;
 import ghidra.framework.plugintool.annotation.AutoServiceConsumed;
+import ghidra.framework.store.local.LocalFileSystem;
 import ghidra.program.model.address.*;
 import ghidra.program.model.lang.*;
 import ghidra.program.util.DefaultLanguageService;
@@ -60,12 +62,14 @@ import ghidra.trace.model.target.path.*;
 import ghidra.trace.model.target.schema.TraceObjectSchema.SchemaName;
 import ghidra.trace.model.target.schema.XmlSchemaContext;
 import ghidra.trace.model.time.TraceSnapshot;
+import ghidra.trace.model.time.schedule.TraceSchedule;
+import ghidra.trace.model.time.schedule.TraceSchedule.TimeRadix;
 import ghidra.util.*;
 import ghidra.util.exception.CancelledException;
 import ghidra.util.exception.DuplicateFileException;
 
 public class TraceRmiHandler extends AbstractTraceRmiConnection {
-	public static final String VERSION = "11.3";
+	public static final String VERSION = "11.4";
 
 	protected static class VersionMismatchError extends TraceRmiError {
 		public VersionMismatchError(String remote) {
@@ -89,9 +93,15 @@ public class TraceRmiHandler extends AbstractTraceRmiConnection {
 	}
 
 	protected static class InvalidObjPathError extends TraceRmiError {
+		public InvalidObjPathError(String path) {
+			super(path);
+		}
 	}
 
 	protected static class NoSuchAddressSpaceError extends TraceRmiError {
+		public NoSuchAddressSpaceError(String name) {
+			super(name);
+		}
 	}
 
 	protected static class InvalidSchemaError extends TraceRmiError {
@@ -291,8 +301,8 @@ public class TraceRmiHandler extends AbstractTraceRmiConnection {
 		synchronized (openTxes) {
 			while (!openTxes.isEmpty()) {
 				Tid nextKey = openTxes.keySet().iterator().next();
-				OpenTx open = openTxes.remove(nextKey);
-				open.tx.close();
+				OpenTx openTx = openTxes.remove(nextKey);
+				openTx.tx.close();
 			}
 		}
 
@@ -308,7 +318,7 @@ public class TraceRmiHandler extends AbstractTraceRmiConnection {
 					// OK. Move on
 				}
 			}
-			open.trace.release(this);
+			open.dispose(this);
 		}
 		closed.complete(null);
 		plugin.listeners.invoke().disconnected(this);
@@ -733,77 +743,43 @@ public class TraceRmiHandler extends AbstractTraceRmiConnection {
 	}
 
 	protected static Value makeValue(Object value) {
-		if (value instanceof Void) {
-			return Value.newBuilder().setNullValue(Null.getDefaultInstance()).build();
-		}
-		if (value instanceof Boolean b) {
-			return Value.newBuilder().setBoolValue(b).build();
-		}
-		if (value instanceof Byte b) {
-			return Value.newBuilder().setByteValue(b).build();
-		}
-		if (value instanceof Character c) {
-			return Value.newBuilder().setCharValue(c).build();
-		}
-		if (value instanceof Short s) {
-			return Value.newBuilder().setShortValue(s).build();
-		}
-		if (value instanceof Integer i) {
-			return Value.newBuilder().setIntValue(i).build();
-		}
-		if (value instanceof Long l) {
-			return Value.newBuilder().setLongValue(l).build();
-		}
-		if (value instanceof String s) {
-			return Value.newBuilder().setStringValue(s).build();
-		}
-		if (value instanceof boolean[] ba) {
-			return Value.newBuilder()
+		return switch (value) {
+			case Void v -> Value.newBuilder().setNullValue(Null.getDefaultInstance()).build();
+			case Boolean b -> Value.newBuilder().setBoolValue(b).build();
+			case Byte b -> Value.newBuilder().setByteValue(b).build();
+			case Character c -> Value.newBuilder().setCharValue(c).build();
+			case Short s -> Value.newBuilder().setShortValue(s).build();
+			case Integer i -> Value.newBuilder().setIntValue(i).build();
+			case Long l -> Value.newBuilder().setLongValue(l).build();
+			case String s -> Value.newBuilder().setStringValue(s).build();
+			case boolean[] ba -> Value.newBuilder()
 					.setBoolArrValue(
 						BoolArr.newBuilder().addAllArr(Arrays.asList(ArrayUtils.toObject(ba))))
 					.build();
-		}
-		if (value instanceof byte[] ba) {
-			return Value.newBuilder().setBytesValue(ByteString.copyFrom(ba)).build();
-		}
-		if (value instanceof char[] ca) {
-			return Value.newBuilder().setCharArrValue(new String(ca)).build();
-		}
-		if (value instanceof short[] sa) {
-			return Value.newBuilder()
+			case byte[] ba -> Value.newBuilder().setBytesValue(ByteString.copyFrom(ba)).build();
+			case char[] ca -> Value.newBuilder().setCharArrValue(new String(ca)).build();
+			case short[] sa -> Value.newBuilder()
 					.setShortArrValue(ShortArr.newBuilder()
 							.addAllArr(
 								Stream.of(ArrayUtils.toObject(sa)).map(s -> (int) s).toList()))
 					.build();
-		}
-		if (value instanceof int[] ia) {
-			return Value.newBuilder()
+			case int[] ia -> Value.newBuilder()
 					.setIntArrValue(
 						IntArr.newBuilder().addAllArr(IntStream.of(ia).mapToObj(i -> i).toList()))
 					.build();
-		}
-		if (value instanceof long[] la) {
-			return Value.newBuilder()
+			case long[] la -> Value.newBuilder()
 					.setLongArrValue(
 						LongArr.newBuilder().addAllArr(LongStream.of(la).mapToObj(l -> l).toList()))
 					.build();
-		}
-		if (value instanceof String[] sa) {
-			return Value.newBuilder()
+			case String[] sa -> Value.newBuilder()
 					.setStringArrValue(StringArr.newBuilder().addAllArr(List.of(sa)))
 					.build();
-		}
-		if (value instanceof Address a) {
-			return Value.newBuilder().setAddressValue(makeAddr(a)).build();
-		}
-		if (value instanceof AddressRange r) {
-			return Value.newBuilder().setRangeValue(makeAddrRange(r)).build();
-		}
-		if (value instanceof TraceObject o) {
-			return Value.newBuilder().setChildDesc(makeObjDesc(o)).build();
-		}
-		throw new AssertionError(
-			"Cannot encode value: " + value + "(type=" + value.getClass() + ")");
+			case Address a -> Value.newBuilder().setAddressValue(makeAddr(a)).build();
+			case AddressRange r -> Value.newBuilder().setRangeValue(makeAddrRange(r)).build();
+			case TraceObject o -> Value.newBuilder().setChildDesc(makeObjDesc(o)).build();
+			default -> throw new AssertionError(
+				"Cannot encode value: " + value + "(type=" + value.getClass() + ")");
+		};
 	}
 
 	protected static MethodArgument makeArgument(String name, Object value) {
@@ -839,7 +815,7 @@ public class TraceRmiHandler extends AbstractTraceRmiConnection {
 	protected ReplyCloseTrace handleCloseTrace(RequestCloseTrace req) {
 		OpenTrace open = requireOpenTrace(req.getOid());
 		openTraces.removeById(open.doId);
-		open.trace.release(this);
+		open.dispose(this);
 		return ReplyCloseTrace.getDefaultInstance();
 	}
 
@@ -884,11 +860,24 @@ public class TraceRmiHandler extends AbstractTraceRmiConnection {
 		DoId doId = requireAvailableDoId(req.getOid());
 		openTraces.put(new OpenTrace(doId, trace, target));
 		createDeconflictedFile(folder, trace);
+		doActivate(null, trace, null);
 		return ReplyCreateTrace.getDefaultInstance();
 	}
 
+	protected static String sanitizeName(String name) {
+		StringBuffer buf = new StringBuffer(name.length());
+		for (int i = 0; i < name.length(); i++) {
+			char c = name.charAt(i);
+			buf.append(LocalFileSystem.isValidNameCharacter(c) ? c : '_');
+		}
+		return buf.toString();
+	}
+
 	protected static List<String> sanitizePath(String path) {
-		return Stream.of(path.split("\\\\|/")).filter(p -> !p.isBlank()).toList();
+		return Stream.of(path.split("\\\\|/"))
+				.filter(n -> !n.isBlank())
+				.map(n -> sanitizeName(n))
+				.toList();
 	}
 
 	protected ReplyDeleteBytes handleDeleteBytes(RequestDeleteBytes req)
@@ -931,7 +920,8 @@ public class TraceRmiHandler extends AbstractTraceRmiConnection {
 
 		// Want addresses satisfying {@code known | (readOnly & everKnown)}
 		TraceMemoryManager memoryManager = open.trace.getMemoryManager();
-		AddressSetView readOnly = memoryManager.getRegionsAddressSetWith(snap, r -> !r.isWrite());
+		AddressSetView readOnly =
+			memoryManager.getRegionsAddressSetWith(snap, r -> !r.isWrite(snap));
 		AddressSetView everKnown = memoryManager.getAddressesWithState(Lifespan.since(snap),
 			s -> s == TraceMemoryState.KNOWN);
 		AddressSetView roEverKnown = new IntersectionAddressSetView(readOnly, everKnown);
@@ -950,8 +940,9 @@ public class TraceRmiHandler extends AbstractTraceRmiConnection {
 			dis.applyTo(open.trace.getFixedProgramView(snap), monitor);
 		}
 
+		AddressSetView result = dis.getDisassembledAddressSet();
 		return ReplyDisassemble.newBuilder()
-				.setLength(dis.getDisassembledAddressSet().getNumAddresses())
+				.setLength(result == null ? 0 : result.getNumAddresses())
 				.build();
 	}
 
@@ -967,13 +958,29 @@ public class TraceRmiHandler extends AbstractTraceRmiConnection {
 			Msg.error(this, "Back-end debugger aborted a transaction!");
 			tx.tx.abortOnClose();
 		}
-		tx.tx.close();
+
 		OpenTrace open = requireOpenTrace(tx.txId.doId);
 		if (!tx.undoable) {
-			open.trace.clearUndo();
+			/**
+			 * The listener is invoked via runLater, so we must do the same here, so that events are
+			 * processed in the order emitted.
+			 */
+			Swing.runLater(() -> open.txListener.markNotUndoable());
 		}
-		// TODO: Check for other transactions on the same trace?
-		open.trace.setEventsEnabled(true);
+
+		tx.tx.close();
+
+		final boolean restoreEvents;
+		synchronized (openTxes) {
+			restoreEvents = openTxes.keySet()
+					.stream()
+					.noneMatch(id -> id.doId.domObjId == req.getOid().getId());
+		}
+		if (restoreEvents) {
+			open.trace.setEventsEnabled(true);
+		}
+		Swing.runLater(
+			() -> plugin.listeners.invoke().transactionClosed(this, open.target, req.getAbort()));
 		return ReplyEndTx.getDefaultInstance();
 	}
 
@@ -1027,7 +1034,9 @@ public class TraceRmiHandler extends AbstractTraceRmiConnection {
 		}
 		for (Method m : req.getMethodsList()) {
 			RemoteMethod rm = new RecordRemoteMethod(this, m.getName(),
-				ActionName.name(m.getAction()), m.getDisplay(), m.getDescription(),
+				ActionName.name(m.getAction()), m.getDisplay(),
+				m.getIcon().isBlank() ? null : new GIcon(m.getIcon()), m.getOkText(),
+				m.getDescription(),
 				m.getParametersList()
 						.stream()
 						.collect(Collectors.toMap(MethodParameter::getName, this::makeParameter)),
@@ -1156,13 +1165,21 @@ public class TraceRmiHandler extends AbstractTraceRmiConnection {
 
 	protected ReplySnapshot handleSnapshot(RequestSnapshot req) {
 		OpenTrace open = requireOpenTrace(req.getOid());
-		TraceSnapshot snapshot = open.createSnapshot(req.getSnap(), req.getDescription());
+		TraceSnapshot snapshot = switch (req.getTimeCase()) {
+			case TIME_NOT_SET -> throw new TraceRmiError("snap or time required");
+			case SNAP -> open.createSnapshot(req.getSnap().getSnap());
+			case SCHEDULE -> open.createSnapshot(
+				TraceSchedule.parse(req.getSchedule().getSchedule(), TimeRadix.DEC));
+		};
+		snapshot.setDescription(req.getDescription());
 		if (!"".equals(req.getDatetime())) {
 			Instant instant =
 				DateTimeFormatter.ISO_INSTANT.parse(req.getDatetime()).query(Instant::from);
 			snapshot.setRealTime(instant.toEpochMilli());
 		}
-		return ReplySnapshot.getDefaultInstance();
+		return ReplySnapshot.newBuilder()
+				.setSnap(Snap.newBuilder().setSnap(snapshot.getKey()))
+				.build();
 	}
 
 	protected ReplyStartTx handleStartTx(RequestStartTx req) {
@@ -1175,6 +1192,7 @@ public class TraceRmiHandler extends AbstractTraceRmiConnection {
 		synchronized (openTxes) {
 			openTxes.put(tx.txId, tx);
 		}
+		Swing.runLater(() -> plugin.listeners.invoke().transactionOpened(this, open.target));
 		return ReplyStartTx.getDefaultInstance();
 	}
 
@@ -1275,7 +1293,7 @@ public class TraceRmiHandler extends AbstractTraceRmiConnection {
 	@Override
 	public void forceCloseTrace(Trace trace) {
 		OpenTrace open = openTraces.removeByTrace(trace);
-		open.trace.release(this);
+		open.dispose(this);
 	}
 
 	@Override
@@ -1304,4 +1322,24 @@ public class TraceRmiHandler extends AbstractTraceRmiConnection {
 		}
 		return description;
 	}
+
+	@Override
+	public boolean isBusy() {
+		return !openTxes.isEmpty();
+	}
+
+	@Override
+	public boolean isBusy(Target target) {
+		OpenTrace openTrace = openTraces.getByTrace(target.getTrace());
+		if (openTrace == null || openTrace.target != target) {
+			return false;
+		}
+
+		for (Tid tid : openTxes.keySet()) {
+			if (Objects.equals(openTrace.doId, tid.doId)) {
+				return true;
+			}
+		}
+		return false;
+	}
 }
