@@ -26,8 +26,6 @@ import java.util.stream.Collectors;
 
 import javax.swing.*;
 
-import org.apache.commons.lang3.ArrayUtils;
-
 import docking.*;
 import docking.action.*;
 import docking.action.builder.*;
@@ -402,13 +400,20 @@ public class DebuggerModulesProvider extends ComponentProviderAdapter
 				return;
 			}
 			couldHaveChanged = false;
-			String infosThisTime = spec.getInfoForObjects(trace);
+			DebuggerTraceManagerService traceManager =
+				tool.getService(DebuggerTraceManagerService.class);
+			if (traceManager == null) {
+				return;
+			}
+			DebuggerCoordinates current = traceManager.getCurrentFor(trace);
+			long snap = current.getSnap();
+			String infosThisTime = spec.getInfoForObjects(trace, snap);
 			if (Objects.equals(infosThisTime, infosLastTime)) {
 				return;
 			}
 			infosLastTime = infosThisTime;
 
-			spec.runTask(tool, trace);
+			spec.runTask(tool, trace, snap);
 		}
 
 		public void forceMap() {
@@ -419,12 +424,6 @@ public class DebuggerModulesProvider extends ComponentProviderAdapter
 	}
 
 	protected static Set<TraceModule> getSelectedModules(ActionContext context) {
-		if (context instanceof DebuggerModuleActionContext ctx) {
-			return DebuggerLegacyModulesPanel.getSelectedModulesFromContext(ctx);
-		}
-		if (context instanceof DebuggerSectionActionContext ctx) {
-			return DebuggerLegacySectionsPanel.getSelectedModulesFromContext(ctx);
-		}
 		if (context instanceof DebuggerObjectActionContext ctx) {
 			return DebuggerModulesPanel.getSelectedModulesFromContext(ctx);
 		}
@@ -433,12 +432,6 @@ public class DebuggerModulesProvider extends ComponentProviderAdapter
 
 	protected static Set<TraceSection> getSelectedSections(ActionContext context,
 			boolean allowExpansion) {
-		if (context instanceof DebuggerModuleActionContext ctx) {
-			return DebuggerLegacyModulesPanel.getSelectedSectionsFromContext(ctx);
-		}
-		if (context instanceof DebuggerSectionActionContext ctx) {
-			return DebuggerLegacySectionsPanel.getSelectedSectionsFromContext(ctx, allowExpansion);
-		}
 		if (context instanceof DebuggerObjectActionContext ctx) {
 			return DebuggerModulesPanel.getSelectedSectionsFromContext(ctx);
 		}
@@ -446,12 +439,6 @@ public class DebuggerModulesProvider extends ComponentProviderAdapter
 	}
 
 	protected static AddressSetView getSelectedAddresses(ActionContext context) {
-		if (context instanceof DebuggerModuleActionContext ctx) {
-			return DebuggerLegacyModulesPanel.getSelectedAddressesFromContext(ctx);
-		}
-		if (context instanceof DebuggerSectionActionContext ctx) {
-			return DebuggerLegacySectionsPanel.getSelectedAddressesFromContext(ctx);
-		}
 		if (context instanceof DebuggerObjectActionContext ctx) {
 			return DebuggerModulesPanel.getSelectedAddressesFromContext(ctx);
 		}
@@ -558,9 +545,7 @@ public class DebuggerModulesProvider extends ComponentProviderAdapter
 	private final int defaultDividerSize = mainPanel.getDividerSize();
 
 	DebuggerModulesPanel modulesPanel;
-	DebuggerLegacyModulesPanel legacyModulesPanel;
 	DebuggerSectionsPanel sectionsPanel;
-	DebuggerLegacySectionsPanel legacySectionsPanel;
 
 	// TODO: Lazy construction of these dialogs?
 	private final DebuggerBlockChooserDialog blockChooserDialog;
@@ -593,7 +578,6 @@ public class DebuggerModulesProvider extends ComponentProviderAdapter
 	boolean filterSectionsByModules = false;
 
 	private final Map<Trace, AutoMapState> autoMapStateByTrace = new WeakHashMap<>();
-	private AutoMapState forMappingListener;
 
 	DockingAction actionImportMissingModule;
 	DockingAction actionMapMissingModule;
@@ -634,7 +618,7 @@ public class DebuggerModulesProvider extends ComponentProviderAdapter
 
 	private void importModuleFromFileSystem(TraceModule module) {
 		GhidraFileChooser chooser = new GhidraFileChooser(getComponent());
-		chooser.setSelectedFile(new File(module.getName()));
+		chooser.setSelectedFile(new File(module.getName(current.getSnap())));
 		File file = chooser.getSelectedFile();
 		chooser.dispose();
 		if (file == null) { // Perhaps cancelled
@@ -707,34 +691,19 @@ public class DebuggerModulesProvider extends ComponentProviderAdapter
 		contextChanged();
 	}
 
-	void legacyModulesPanelContextChanged() {
-		myActionContext = legacyModulesPanel.getActionContext();
-		if (isFilterSectionsByModules()) {
-			legacySectionsPanel.loadSections();
-		}
-		contextChanged();
-	}
-
 	void sectionsPanelContextChanged() {
 		myActionContext = sectionsPanel.getActionContext();
 		contextChanged();
 	}
 
-	void legacySectionsPanelContextChanged() {
-		myActionContext = legacySectionsPanel.getActionContext();
-		contextChanged();
-	}
-
 	protected void buildMainPanel() {
 		mainPanel.setContinuousLayout(true);
 
 		modulesPanel = new DebuggerModulesPanel(this);
 		mainPanel.setLeftComponent(modulesPanel);
-		legacyModulesPanel = new DebuggerLegacyModulesPanel(this);
 
 		sectionsPanel = new DebuggerSectionsPanel(this);
 		mainPanel.setRightComponent(sectionsPanel);
-		legacySectionsPanel = new DebuggerLegacySectionsPanel(this);
 
 		mainPanel.setResizeWeight(0.5);
 	}
@@ -837,7 +806,7 @@ public class DebuggerModulesProvider extends ComponentProviderAdapter
 			return !ctx.getSelectedModules().isEmpty();
 		}
 		if (context instanceof DebuggerSectionActionContext ctx) {
-			return !ctx.getSelectedSections(false).isEmpty();
+			return !ctx.getSelectedSections(false, current.getSnap()).isEmpty();
 		}
 		if (context instanceof DebuggerObjectActionContext ctx) {
 			return !ctx.getObjectValues().isEmpty();
@@ -978,10 +947,13 @@ public class DebuggerModulesProvider extends ComponentProviderAdapter
 		if (staticMappingService == null) {
 			return;
 		}
+
 		Program program = context.getProgram();
 		Trace trace = context.getTrace();
+		long snap = traceManager.getCurrentFor(trace).getSnap();
+
 		Map<TraceModule, ModuleMapProposal> map = staticMappingService.proposeModuleMaps(
-			trace.getModuleManager().getAllModules(), List.of(program));
+			trace.getModuleManager().getAllModules(), snap, List.of(program));
 		Collection<ModuleMapEntry> proposal = MapProposal.flatten(map.values());
 		promptModuleProposal(proposal, FMT_NO_MODULES_PROPOSAL_RETRY.formatted(
 			trace.getDomainFile().getName(), program.getDomainFile().getName()));
@@ -1022,10 +994,10 @@ public class DebuggerModulesProvider extends ComponentProviderAdapter
 
 		Program program = context.getProgram();
 		ModuleMapProposal proposal =
-			staticMappingService.proposeModuleMap(module, program);
+			staticMappingService.proposeModuleMap(module, snap, program);
 		Map<TraceModule, ModuleMapEntry> map = proposal.computeMap();
 		promptModuleProposal(map.values(), FMT_NO_MODULES_PROPOSAL_CURRENT.formatted(
-			module.getName(), program.getDomainFile().getName()));
+			module.getName(snap), program.getDomainFile().getName()));
 	}
 
 	private void activatedMapMissingProgramIdentically(
@@ -1062,7 +1034,6 @@ public class DebuggerModulesProvider extends ComponentProviderAdapter
 		actionShowSectionsTable.setSelected(showSectionsTable);
 		mainPanel.setDividerSize(showSectionsTable ? defaultDividerSize : 0);
 		sectionsPanel.setVisible(showSectionsTable);
-		legacySectionsPanel.setVisible(showSectionsTable);
 		mainPanel.resetToPreferredSizes();
 	}
 
@@ -1081,7 +1052,6 @@ public class DebuggerModulesProvider extends ComponentProviderAdapter
 		this.filterSectionsByModules = filterSectionsByModules;
 		actionFilterSectionsByModules.setSelected(filterSectionsByModules);
 		sectionsPanel.setFilteredBySelectedModules(filterSectionsByModules);
-		legacySectionsPanel.setFilteredBySelectedModules(filterSectionsByModules);
 	}
 
 	private void activatedSelectCurrent(ActionContext ignored) {
@@ -1089,16 +1059,16 @@ public class DebuggerModulesProvider extends ComponentProviderAdapter
 			return;
 		}
 
+		long snap = current.getSnap();
 		ProgramSelection progSel = listingService.getCurrentSelection();
 		TraceModuleManager moduleManager = current.getTrace().getModuleManager();
 		if (progSel != null && !progSel.isEmpty()) {
-			long snap = traceManager.getCurrentSnap();
 			Set<TraceModule> modSel = new HashSet<>();
 			Set<TraceSection> sectionSel = new HashSet<>();
 			for (AddressRange range : progSel) {
 				for (TraceModule module : moduleManager
 						.getModulesIntersecting(Lifespan.at(snap), range)) {
-					if (module.getSections().isEmpty()) {
+					if (module.getSections(snap).isEmpty()) {
 						modSel.add(module);
 					}
 				}
@@ -1127,7 +1097,7 @@ public class DebuggerModulesProvider extends ComponentProviderAdapter
 			TraceModule bestModule = null;
 			for (TraceModule module : moduleManager
 					.getLoadedModules(traceManager.getCurrentSnap())) {
-				Address base = module.getBase();
+				Address base = module.getBase(snap);
 				if (base == null || base.getAddressSpace() != address.getAddressSpace()) {
 					continue;
 				}
@@ -1138,12 +1108,12 @@ public class DebuggerModulesProvider extends ComponentProviderAdapter
 				if (base.compareTo(address) > 0) {
 					continue;
 				}
-				if (base.compareTo(bestModule.getBase()) <= 0) {
+				if (base.compareTo(bestModule.getBase(snap)) <= 0) {
 					continue;
 				}
 				bestModule = module;
 			}
-			if (bestModule.getSections().isEmpty()) {
+			if (bestModule.getSections(snap).isEmpty()) {
 				setSelectedModules(Set.of(bestModule));
 				return;
 			}
@@ -1168,8 +1138,9 @@ public class DebuggerModulesProvider extends ComponentProviderAdapter
 		if (staticMappingService == null) {
 			return;
 		}
-		Map<TraceModule, ModuleMapProposal> map = staticMappingService.proposeModuleMaps(modules,
-			List.of(programManager.getAllOpenPrograms()));
+		Map<TraceModule, ModuleMapProposal> map =
+			staticMappingService.proposeModuleMaps(modules, current.getSnap(),
+				List.of(programManager.getAllOpenPrograms()));
 		Collection<ModuleMapEntry> proposal = MapProposal.flatten(map.values());
 		promptModuleProposal(proposal, NO_MODULES_PROPOSAL_SEL);
 	}
@@ -1182,7 +1153,8 @@ public class DebuggerModulesProvider extends ComponentProviderAdapter
 		if (program == null) {
 			return;
 		}
-		ModuleMapProposal proposal = staticMappingService.proposeModuleMap(module, program);
+		ModuleMapProposal proposal =
+			staticMappingService.proposeModuleMap(module, current.getSnap(), program);
 		Map<TraceModule, ModuleMapEntry> map = proposal.computeMap();
 		promptModuleProposal(map.values(), NO_MODULES_PROPOSAL_SEL);
 	}
@@ -1209,8 +1181,9 @@ public class DebuggerModulesProvider extends ComponentProviderAdapter
 		}
 		Set<TraceModule> modules =
 			sections.stream().map(TraceSection::getModule).collect(Collectors.toSet());
-		Map<?, SectionMapProposal> map = staticMappingService.proposeSectionMaps(modules,
-			List.of(programManager.getAllOpenPrograms()));
+		Map<?, SectionMapProposal> map =
+			staticMappingService.proposeSectionMaps(modules, current.getSnap(),
+				List.of(programManager.getAllOpenPrograms()));
 		Collection<SectionMapEntry> proposal = MapProposal.flatten(map.values());
 		Collection<SectionMapEntry> filtered = proposal.stream()
 				.filter(e -> sections.contains(e.getSection()))
@@ -1232,7 +1205,8 @@ public class DebuggerModulesProvider extends ComponentProviderAdapter
 			return;
 		}
 		TraceModule module = modules.iterator().next();
-		SectionMapProposal map = staticMappingService.proposeSectionMap(module, program);
+		SectionMapProposal map =
+			staticMappingService.proposeSectionMap(module, current.getSnap(), program);
 		Collection<SectionMapEntry> proposal = map.computeMap().values();
 		Collection<SectionMapEntry> filtered = proposal.stream()
 				.filter(e -> sections.contains(e.getSection()))
@@ -1249,8 +1223,8 @@ public class DebuggerModulesProvider extends ComponentProviderAdapter
 		if (block == null) {
 			return;
 		}
-		SectionMapProposal map =
-			staticMappingService.proposeSectionMap(section, location.getProgram(), block);
+		SectionMapProposal map = staticMappingService.proposeSectionMap(section, current.getSnap(),
+			location.getProgram(), block);
 		promptSectionProposal(map.computeMap().values());
 	}
 
@@ -1374,52 +1348,18 @@ public class DebuggerModulesProvider extends ComponentProviderAdapter
 			setAutoMapSpec(amState.spec);
 		}
 
-		if (Trace.isLegacy(newTrace)) {
-			modulesPanel.coordinatesActivated(DebuggerCoordinates.NOWHERE);
-			sectionsPanel.coordinatesActivated(DebuggerCoordinates.NOWHERE);
-			legacyModulesPanel.coordinatesActivated(coordinates);
-			legacySectionsPanel.coordinatesActivated(coordinates);
-			if (ArrayUtils.indexOf(mainPanel.getComponents(), legacyModulesPanel) == -1) {
-				mainPanel.remove(modulesPanel);
-				mainPanel.remove(sectionsPanel);
-				mainPanel.setLeftComponent(legacyModulesPanel);
-				mainPanel.setRightComponent(legacySectionsPanel);
-				mainPanel.validate();
-			}
-		}
-		else {
-			legacyModulesPanel.coordinatesActivated(DebuggerCoordinates.NOWHERE);
-			legacySectionsPanel.coordinatesActivated(DebuggerCoordinates.NOWHERE);
-			modulesPanel.coordinatesActivated(coordinates);
-			sectionsPanel.coordinatesActivated(coordinates);
-			if (ArrayUtils.indexOf(mainPanel.getComponents(), modulesPanel) == -1) {
-				mainPanel.remove(legacyModulesPanel);
-				mainPanel.remove(legacySectionsPanel);
-				mainPanel.setLeftComponent(modulesPanel);
-				mainPanel.setRightComponent(sectionsPanel);
-				mainPanel.validate();
-			}
-		}
+		modulesPanel.coordinatesActivated(coordinates);
+		sectionsPanel.coordinatesActivated(coordinates);
 
 		contextChanged();
 	}
 
 	public void setSelectedModules(Set<TraceModule> sel) {
-		if (Trace.isLegacy(current.getTrace())) {
-			legacyModulesPanel.setSelectedModules(sel);
-		}
-		else {
-			modulesPanel.setSelectedModules(sel);
-		}
+		modulesPanel.setSelectedModules(sel);
 	}
 
 	public void setSelectedSections(Set<TraceSection> sel) {
-		if (Trace.isLegacy(current.getTrace())) {
-			legacySectionsPanel.setSelectedSections(sel);
-		}
-		else {
-			sectionsPanel.setSelectedSections(sel);
-		}
+		sectionsPanel.setSelectedSections(sel);
 	}
 
 	private DataTreeDialog getProgramChooserDialog() {
@@ -1444,10 +1384,11 @@ public class DebuggerModulesProvider extends ComponentProviderAdapter
 			Msg.warn(this, "No program manager!");
 			return null;
 		}
-		return blockChooserDialog.chooseBlock(getTool(), section,
+		return blockChooserDialog.chooseBlock(getTool(), section, current.getSnap(),
 			List.of(programManager.getAllOpenPrograms()));
 	}
 
+	@Override
 	public void setAutoMapSpec(AutoMapSpec spec) {
 		actionAutoMap.setCurrentActionStateByUserData(spec);
 	}
