@@ -634,27 +634,6 @@ public class DebuggerEmulationServicePlugin extends Plugin implements DebuggerEm
 		return task.future;
 	}
 
-	protected TraceSnapshot findScratch(Trace trace, TraceSchedule time) {
-		Collection<? extends TraceSnapshot> exist =
-			trace.getTimeManager().getSnapshotsWithSchedule(time);
-		if (!exist.isEmpty()) {
-			return exist.iterator().next();
-		}
-		/**
-		 * TODO: This could be more sophisticated.... Does it need to be, though? Ideally, we'd only
-		 * keep state around that has annotations, e.g., bookmarks and code units. That needs a new
-		 * query (latestStartSince) on those managers, though. It must find the latest start tick
-		 * since a given snap. We consider only start snaps because placed code units go "from now
-		 * on out".
-		 */
-		TraceSnapshot last = trace.getTimeManager().getMostRecentSnapshot(-1);
-		long snap = last == null ? Long.MIN_VALUE : last.getKey() + 1;
-		TraceSnapshot snapshot = trace.getTimeManager().getSnapshot(snap, true);
-		snapshot.setDescription("Emulated");
-		snapshot.setSchedule(time);
-		return snapshot;
-	}
-
 	protected void installBreakpoints(Trace trace, long snap, DebuggerPcodeMachine<?> emu) {
 		Lifespan span = Lifespan.at(snap);
 		TraceBreakpointManager bm = trace.getBreakpointManager();
@@ -664,30 +643,30 @@ public class DebuggerEmulationServicePlugin extends Plugin implements DebuggerEm
 				if (!bpt.isEmuEnabled(snap)) {
 					continue;
 				}
-				Set<TraceBreakpointKind> kinds = bpt.getKinds();
+				Set<TraceBreakpointKind> kinds = bpt.getKinds(snap);
 				boolean isExecute =
 					kinds.contains(TraceBreakpointKind.HW_EXECUTE) ||
 						kinds.contains(TraceBreakpointKind.SW_EXECUTE);
 				boolean isRead = kinds.contains(TraceBreakpointKind.READ);
 				boolean isWrite = kinds.contains(TraceBreakpointKind.WRITE);
 				if (isExecute) {
+					Address minAddress = bpt.getMinAddress(snap);
 					try {
-						emu.inject(bpt.getMinAddress(), bpt.getEmuSleigh());
+						emu.inject(minAddress, bpt.getEmuSleigh(snap));
 					}
 					catch (Exception e) { // This is a bit broad...
-						Msg.error(this,
-							"Error compiling breakpoint Sleigh at " + bpt.getMinAddress(), e);
-						emu.inject(bpt.getMinAddress(), "emu_injection_err();");
+						Msg.error(this, "Error compiling breakpoint Sleigh at " + minAddress, e);
+						emu.inject(minAddress, "emu_injection_err();");
 					}
 				}
 				if (isRead && isWrite) {
-					emu.addAccessBreakpoint(bpt.getRange(), AccessKind.RW);
+					emu.addAccessBreakpoint(bpt.getRange(snap), AccessKind.RW);
 				}
 				else if (isRead) {
-					emu.addAccessBreakpoint(bpt.getRange(), AccessKind.R);
+					emu.addAccessBreakpoint(bpt.getRange(snap), AccessKind.R);
 				}
 				else if (isWrite) {
-					emu.addAccessBreakpoint(bpt.getRange(), AccessKind.W);
+					emu.addAccessBreakpoint(bpt.getRange(snap), AccessKind.W);
 				}
 			}
 		}
@@ -753,7 +732,8 @@ public class DebuggerEmulationServicePlugin extends Plugin implements DebuggerEm
 	protected TraceSnapshot writeToScratch(CacheKey key, CachedEmulator ce) {
 		TraceSnapshot destSnap;
 		try (Transaction tx = key.trace.openTransaction("Emulate")) {
-			destSnap = findScratch(key.trace, key.time);
+			destSnap = key.trace.getTimeManager().findScratchSnapshot(key.time);
+			destSnap.setDescription("Emulated");
 			try {
 				ce.emulator().writeDown(key.platform, destSnap.getKey(), key.time.getSnap());
 			}
