@@ -4,9 +4,9 @@
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,6 +15,8 @@
  */
 package ghidra.app.cmd.function;
 
+import static ghidra.program.model.symbol.RefType.*;
+
 import java.math.BigInteger;
 import java.util.*;
 import java.util.concurrent.atomic.AtomicInteger;
@@ -732,36 +734,66 @@ public class CreateThunkFunctionCmd extends BackgroundCommand<Program> {
 			FlowType flowType, boolean checkForSideEffects, HashSet<Varnode> setRegisters,
 			HashSet<Varnode> usedRegisters) {
 
-		// conditional jumps can't be thunks.
-		// any other flow, not good
-		Address flowingAddr = null;
-		if ((flowType.isJump() || flowType.equals(RefType.COMPUTED_CALL_TERMINATOR) ||
-			flowType.equals(RefType.CALL_TERMINATOR)) && !flowType.isConditional()) {
-			// program counter should be assumed to be used
-
-			// assume PC is used when considering registers that have been set
-			Register PC = program.getLanguage().getProgramCounter();
-			if (PC != null) {
-				usedRegisters.add(new Varnode(PC.getAddress(), PC.getMinimumByteSize()));
+		// conditional jumps/call terminators can't be thunks,
+		// unless not checkingForSideEffects and just trying to get possible thunk address.
+		
+		boolean isJump = flowType.isJump();
+		boolean isCall = flowType.isCall();
+		boolean isConditional = flowType.isConditional();
+		
+		// only jump/call are allowed
+		if (!(isJump || isCall)) {
+			return null;
+		}
+		
+		// no conditional jumps allowed
+		if (isJump && isConditional) {
+			return null;
+		}
+		
+		if (isCall) {
+			// Any conditional call considered as having side-effects
+			if (isConditional && checkForSideEffects) {
+				return null;
 			}
-			setRegisters.removeAll(usedRegisters);
-
-			// check that the setRegisters are all hidden, meaning don't care.
-			for (Iterator<Varnode> iterator = setRegisters.iterator(); iterator.hasNext();) {
-				Varnode rvnode = iterator.next();
-				Register reg = program.getRegister(rvnode);
-				// the register pcode access could have fallen in the middle of a valid register
-				//  thus no register will exist at the varnode
-				if (reg != null && reg.isHidden()) {
-					iterator.remove();
+			// CALL_TERMINATOR, COMPUTED_CALL_TERMINATOR
+			else if (!flowType.isTerminal()) {
+				if (flowType == COMPUTED_CALL ||
+					flowType == UNCONDITIONAL_CALL) {
+					// consider any simple call having side-effects
+					if (checkForSideEffects) {
+						return null;
+					}
 				}
 			}
+		}
+		
+		// program counter should be assumed to be used
+		// assume PC is used when considering registers that have been set
+		Register PC = program.getLanguage().getProgramCounter();
+		if (PC != null) {
+			usedRegisters.add(new Varnode(PC.getAddress(), PC.getMinimumByteSize()));
+		}
+		setRegisters.removeAll(usedRegisters);
 
-			// if not checking for sideEffect registers set, or there are no side-effects
-			if (!checkForSideEffects || setRegisters.size() == 0) {
-				flowingAddr = getFlowingAddress(program, instr);
+		// check that the setRegisters are all hidden, meaning don't care.
+		for (Iterator<Varnode> iterator = setRegisters.iterator(); iterator.hasNext();) {
+			Varnode rvnode = iterator.next();
+			Register reg = program.getRegister(rvnode);
+			// the register pcode access could have fallen in the middle of a valid register
+			//  thus no register will exist at the varnode
+			if (reg != null && reg.isHidden()) {
+				iterator.remove();
 			}
 		}
+
+		Address flowingAddr = null;
+		
+		// if not checking for sideEffect registers set, or there are no side-effects
+		if (!checkForSideEffects || setRegisters.size() == 0) {
+			flowingAddr = getFlowingAddress(program, instr);
+		}
+		
 		return flowingAddr;
 	}
 
