@@ -21,6 +21,7 @@ import docking.action.builder.ActionBuilder;
 import docking.tool.ToolConstants;
 import generic.theme.GIcon;
 import ghidra.app.CorePluginPackage;
+import ghidra.app.context.ListingActionContext;
 import ghidra.app.plugin.PluginCategoryNames;
 import ghidra.app.plugin.core.codebrowser.SelectEndpointsAction.RangeEndpoint;
 import ghidra.app.plugin.core.table.TableComponentProvider;
@@ -81,7 +82,7 @@ public class CodeBrowserSelectionPlugin extends Plugin {
 				.menuGroup(SELECT_GROUP, "a")
 				.keyBinding("ctrl A")
 				.helpLocation(new HelpLocation(HelpTopics.SELECTION, "Select All"))
-				.withContext(CodeViewerActionContext.class, true)
+				.withContext(ListingActionContext.class, true)
 				.inWindow(ActionBuilder.When.CONTEXT_MATCHES)
 				.onAction(c -> ((CodeViewerProvider) c.getComponentProvider()).selectAll())
 				.buildAndInstall(tool);
@@ -90,8 +91,9 @@ public class CodeBrowserSelectionPlugin extends Plugin {
 				.menuPath(ToolConstants.MENU_SELECTION, "&Clear Selection")
 				.menuGroup(SELECT_GROUP, "b")
 				.helpLocation(new HelpLocation(HelpTopics.SELECTION, "Clear Selection"))
-				.withContext(CodeViewerActionContext.class, true)
+				.withContext(ListingActionContext.class, true)
 				.inWindow(ActionBuilder.When.CONTEXT_MATCHES)
+				.enabledWhen(c -> c.hasSelection())
 				.onAction(c -> ((CodeViewerProvider) c.getComponentProvider())
 						.setSelection(new ProgramSelection()))
 				.buildAndInstall(tool);
@@ -100,7 +102,7 @@ public class CodeBrowserSelectionPlugin extends Plugin {
 				.menuPath(ToolConstants.MENU_SELECTION, "&Complement")
 				.menuGroup(SELECT_GROUP, "c")
 				.helpLocation(new HelpLocation(HelpTopics.SELECTION, "Select Complement"))
-				.withContext(CodeViewerActionContext.class, true)
+				.withContext(ListingActionContext.class, true)
 				.inWindow(ActionBuilder.When.CONTEXT_MATCHES)
 				.onAction(c -> ((CodeViewerProvider) c.getComponentProvider()).selectComplement())
 				.buildAndInstall(tool);
@@ -111,7 +113,7 @@ public class CodeBrowserSelectionPlugin extends Plugin {
 				.menuPath(ToolConstants.MENU_SELECTION, "Create Table From Selection")
 				.menuGroup("SelectUtils")
 				.helpLocation(new HelpLocation(HelpTopics.CODE_BROWSER, "Selection_Tables"))
-				.withContext(CodeViewerActionContext.class, true)
+				.withContext(ListingActionContext.class, true)
 				.inWindow(ActionBuilder.When.CONTEXT_MATCHES)
 				.onAction(c -> createTable((CodeViewerProvider) c.getComponentProvider()))
 				.buildAndInstall(tool);
@@ -120,7 +122,7 @@ public class CodeBrowserSelectionPlugin extends Plugin {
 				.menuPath(ToolConstants.MENU_SELECTION, CREATE_ADDRESS_RANGE_TABLE_ACTION_NAME)
 				.menuGroup("SelectUtils")
 				.helpLocation(new HelpLocation(HelpTopics.CODE_BROWSER, "Selection_Tables"))
-				.withContext(CodeViewerActionContext.class, true)
+				.withContext(ListingActionContext.class, true)
 				.inWindow(ActionBuilder.When.CONTEXT_MATCHES)
 				.onAction(
 					c -> createAddressRangeTable((CodeViewerProvider) c.getComponentProvider()))
@@ -149,7 +151,7 @@ public class CodeBrowserSelectionPlugin extends Plugin {
 		String title = "Selected Ranges in " + program.getName();
 		TableComponentProvider<AddressRangeInfo> tableProvider =
 			tableService.showTableWithMarkers(title, "Address Ranges", model,
-				SearchConstants.SEARCH_HIGHLIGHT_COLOR, markerIcon, title, null);
+				SearchConstants.SEARCH_HIGHLIGHT_COLOR, markerIcon, title, componentProvider);
 		tableProvider.installRemoveItemsAction();
 
 		SelectEndpointsAction selectMin =
@@ -185,7 +187,7 @@ public class CodeBrowserSelectionPlugin extends Plugin {
 		Icon markerIcon = new GIcon("icon.plugin.codebrowser.cursor.marker");
 		TableComponentProvider<Address> tableProvider =
 			tableService.showTableWithMarkers(title + " " + model.getName(), "Selections", model,
-				SearchConstants.SEARCH_HIGHLIGHT_COLOR, markerIcon, title, null);
+				SearchConstants.SEARCH_HIGHLIGHT_COLOR, markerIcon, title, componentProvider);
 		tableProvider.installRemoveItemsAction();
 	}
 
