@@ -338,7 +338,7 @@ public class ProgramBuilder {
 	 * Sets the bytes starting at {@code address} to the values encoded in {@code byteString}.
 	 * <p>
 	 * See {@link #setBytes(String, byte[], boolean)}.
-	 * <p>
+	 * 
 	 * @param address String containing numeric value, preferably hex encoded: "0x1004000"
 	 * @param byteString String containing 2 digit hex values, separated by ' ' space chars
 	 * or by comma ',' chars: "12 05 ff".  See {@link NumericUtilities#parseHexLong(String)}.
@@ -354,7 +354,7 @@ public class ProgramBuilder {
 	 * and then optionally disassembling.
 	 * <p>
 	 * See {@link #setBytes(String, byte[], boolean)}.
-	 * <p>
+	 * 
 	 * @param address String containing numeric value, preferably hex encoded: "0x1004000"
 	 * @param byteString String containing 2 digit hex values, separated by ' ' space chars
 	 * or by comma ',' chars: "12 05 ff".  See {@link NumericUtilities#parseHexLong(String)}.
@@ -373,7 +373,7 @@ public class ProgramBuilder {
 	/**
 	 * Sets the bytes starting at {@code stringAddress} to the byte values in {@code bytes}
 	 * and then optionally disassembling.
-	 * <p>
+	 * 
 	 * @param stringAddress String containing numeric value, preferably hex encoded: "0x1004000"
 	 * @param bytes array of bytes to copy into the memory buffer at the addresss.
 	 * @param disassemble boolean flag.  See {@link #disassemble(String, int)}
@@ -639,7 +639,7 @@ public class ProgramBuilder {
 		});
 	}
 
-	public Namespace createClassNamespace(String name, String parentNamespace, SourceType type)
+	public GhidraClass createClassNamespace(String name, String parentNamespace, SourceType type)
 			throws Exception {
 		return tx(() -> {
 			Namespace ns = getNamespace(parentNamespace);
@@ -933,6 +933,7 @@ public class ProgramBuilder {
 		});
 	}
 
+	@Deprecated(forRemoval = true, since = "11.4")
 	public void createComment(String address, String comment, int commentType) {
 		tx(() -> {
 			Listing listing = program.getListing();
@@ -940,6 +941,13 @@ public class ProgramBuilder {
 		});
 	}
 
+	public void createComment(String address, String comment, CommentType commentType) {
+		tx(() -> {
+			Listing listing = program.getListing();
+			listing.setComment(addr(address), commentType, comment);
+		});
+	}
+
 	public void createFunctionComment(String entryPointAddress, String comment) {
 		tx(() -> {
 			FunctionManager functionManager = program.getFunctionManager();
@@ -1104,10 +1112,9 @@ public class ProgramBuilder {
 		}
 
 		return tx(() -> {
-			FileBytes fileBytes =
-				program.getMemory()
-						.createFileBytes("test", 0, size, new ByteArrayInputStream(bytes),
-							TaskMonitor.DUMMY);
+			FileBytes fileBytes = program.getMemory()
+					.createFileBytes("test", 0, size, new ByteArrayInputStream(bytes),
+						TaskMonitor.DUMMY);
 
 			return fileBytes;
 		});
