@@ -478,6 +478,20 @@ public class DockingWindowManager implements PropertyChangeListener, Placeholder
 		return windowNode.getComponentCount() == 1;
 	}
 
+	/**
+	 * Returns true if the given provider is the last provider in its window.
+	 * @param provider the provider
+	 * @return true if the given provider is the last provider in its window.
+	 */
+	public boolean isLastComponentInWindow(ComponentProvider provider) {
+		Window providerWindow = getProviderWindow(provider);
+		WindowNode providerNode = root.getNodeForWindow(providerWindow);
+		if (providerNode != null) {
+			return providerNode.getComponentCount() == 1;
+		}
+		return false;
+	}
+
 	/**
 	 * Sets the visible state of the set of docking windows.
 	 *
@@ -2228,7 +2242,6 @@ public class DockingWindowManager implements PropertyChangeListener, Placeholder
 	 * be grouped with a specific set of actions.
 	 * <p>
 	 * The default group for a cascaded submenu is the name of the submenu.
-	 * <p>
 	 *
 	 * @param menuPath menu name path where the last element corresponds to the specified group
 	 *            name.
