@@ -4,9 +4,9 @@
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -249,7 +249,7 @@ public class ArmAnalyzer extends ConstantPropagationAnalyzer {
 			};
 			
 		eval.setTrustWritableMemory(trustWriteMemOption)
-		    .setMinpeculativeOffset(minSpeculativeRefAddress)
+		    .setMinSpeculativeOffset(minSpeculativeRefAddress)
 		    .setMaxSpeculativeOffset(maxSpeculativeRefAddress)
 		    .setMinStoreLoadOffset(minStoreLoadRefAddress)
 		    .setCreateComplexDataFromPointers(createComplexDataFromPointers);
@@ -799,7 +799,7 @@ public class ArmAnalyzer extends ConstantPropagationAnalyzer {
 		if (tmodeRegister != null && listing.getUndefinedDataAt(addr) != null) {
 			boolean inThumbMode = false;
 			RegisterValue curvalue =
-				context.getRegisterValue(tmodeRegister, instruction.getMinAddress());
+				context.getRegisterValue(tmodeRegister);
 			if (curvalue != null && curvalue.hasValue()) {
 				inThumbMode = (curvalue.getUnsignedValue().intValue() == 1);
 			}
